@@ -41,6 +41,17 @@ module Milter
       end
     end
 
+    def reload
+      reload_callbacks.each do |callback|
+        callback.call(self)
+      end
+    end
+
+    def on_reload(&callback)
+      raise ArgumentError, "callback block on reload is missing" if callback.nil?
+      reload_callbacks << callback
+    end
+
     private
     def setup_session(context, session_class, session_new_arguments)
       session_context = ClientSessionContext.new(context)
@@ -68,6 +79,10 @@ module Milter
         end
       end
     end
+
+    def reload_callbacks
+      @reload_callbacks ||= []
+    end
   end
 
   class ClientSession
@@ -117,8 +132,11 @@ module Milter
       code = options[:code]
       extended_code = options[:extended_code]
       reason = options[:reason]
-      if code
+      if code or extended_code or reason
 	# TODO: validate parameters.
+        code ||= 550
+        extended_code ||= "5.7.1"
+        reason ||= "Command rejected"
         @context.set_reply(code, extended_code, reason)
       else
         @context.status = :reject
@@ -129,8 +147,11 @@ module Milter
       code = options[:code]
       extended_code = options[:extended_code]
       reason = options[:reason]
-      if code
+      if code or extended_code or reason
 	# TODO: validate parameters.
+        code ||= 451
+        extended_code ||= "4.7.1"
+        reason ||= "Service unavailable - try again later"
         @context.set_reply(code, extended_code, reason)
       else
         @context.status = :temporary_failure
