@@ -161,4 +161,20 @@ module Milter
       end
     end
   end
+
+  module MacroNameNormalizer
+    def normalize_macro_name(name)
+      name.sub(/\A\{(.+)\}\z/, '\1')
+    end
+  end
+
+  module MacroPredicates
+    def authenticated?
+      (self["auth_type"] or self["auth_authen"]) ? true : false
+    end
+
+    def postfix?
+      (/\bPostfix\b/i =~ (self["v"] || '')) ? true : false
+    end
+  end
 end
