@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+# Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
 #
 # This library is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -179,6 +179,51 @@ class TestConfigurationLoader < Test::Unit::TestCase
                  @loader.manager.custom_configuration_directory)
   end
 
+  def test_manager_fallback_status
+    assert_equal(Milter::Status::ACCEPT, @configuration.fallback_status)
+    @loader.manager.fallback_status = "reject"
+    assert_equal(Milter::Status::REJECT, @configuration.fallback_status)
+  end
+
+  def test_manager_fallback_status_at_disconnect
+    assert_equal(Milter::Status::TEMPORARY_FAILURE,
+                 @configuration.fallback_status_at_disconnect)
+    @loader.manager.fallback_status_at_disconnect = "reject"
+    assert_equal(Milter::Status::REJECT,
+                 @configuration.fallback_status_at_disconnect)
+  end
+
+  def test_manager_event_loop_backend
+    assert_equal(Milter::Client::EVENT_LOOP_BACKEND_GLIB,
+                 @configuration.event_loop_backend)
+    @loader.manager.event_loop_backend = "libev"
+    assert_equal(Milter::Client::EVENT_LOOP_BACKEND_LIBEV,
+                 @configuration.event_loop_backend)
+
+    @configuration.event_loop_backend = "glib"
+    assert_equal(Milter::Client::EVENT_LOOP_BACKEND_GLIB,
+                 @loader.manager.event_loop_backend)
+    @configuration.event_loop_backend = "libev"
+    assert_equal(Milter::Client::EVENT_LOOP_BACKEND_LIBEV,
+                 @loader.manager.event_loop_backend)
+  end
+
+  def test_manager_n_workers
+    assert_equal(0, @configuration.n_workers)
+    @loader.manager.n_workers = 5
+    assert_equal(5, @configuration.n_workers)
+    @loader.manager.n_workers = nil
+    assert_equal(0, @configuration.n_workers)
+  end
+
+  def test_manager_packet_buffer_size
+    assert_equal(0, @configuration.default_packet_buffer_size)
+    @loader.manager.packet_buffer_size = 4096
+    assert_equal(4096, @configuration.default_packet_buffer_size)
+    @loader.manager.packet_buffer_size = nil
+    assert_equal(0, @configuration.default_packet_buffer_size)
+  end
+
   def test_to_xml
     assert_equal(<<-EOX, @configuration.to_xml)
 <configuration>
