@@ -767,11 +767,13 @@ gboolean             milter_client_is_processing     (MilterClient  *client);
  * milter_client_start_syslog:
  * @client: a %MilterClient.
  * @identify: the identify name in syslog.
+ * @facility: the syslog facility. Maybe %NULL.
  *
  * Starts logging by syslog with @identify name.
  */
 void                 milter_client_start_syslog      (MilterClient  *client,
-                                                      const gchar   *identify);
+                                                      const gchar   *identify,
+                                                      const gchar   *facility);
 
 /**
  * milter_client_get_process_loop:
@@ -876,6 +878,18 @@ void                 milter_client_set_custom_fork_func (MilterClient *client,
 MilterClientCustomForkFunc
                      milter_client_get_custom_fork_func (MilterClient *client);
 
+/**
+ * milter_client_set_worker_id:
+ * @client: a %MilterClient.
+ *
+ * Gets the worker ID of the client. 0 in parent process, 1 or more in
+ * worker processes.
+ *
+ * Returns: the worker ID of the client.
+ */
+guint                milter_client_get_worker_id        (MilterClient *client);
+
+
 G_END_DECLS
 
 #endif /* __MILTER_CLIENT_CLIENT_H__ */
