@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -38,6 +38,7 @@
 typedef struct _MilterAgentPrivate	MilterAgentPrivate;
 struct _MilterAgentPrivate
 {
+    MilterEventLoop *event_loop;
     MilterEncoder *encoder;
     MilterDecoder *decoder;
     MilterReader *reader;
@@ -45,6 +46,7 @@ struct _MilterAgentPrivate
     gboolean finished;
     guint tag;
     GTimer *timer;
+    gboolean shutting_down;
 };
 
 enum
@@ -53,9 +55,18 @@ enum
     PROP_READER,
     PROP_DECODER,
     PROP_TAG,
-    PROP_ELAPSED
+    PROP_ELAPSED,
+    PROP_EVENT_LOOP
 };
 
+enum
+{
+    FLUSHED,
+    LAST_SIGNAL
+};
+
+static gint signals[LAST_SIGNAL] = {0};
+
 static GStaticMutex auto_tag_mutex = G_STATIC_MUTEX_INIT;
 static guint auto_tag = 0;
 
@@ -83,6 +94,9 @@ static void get_property   (GObject         *object,
                             GValue          *value,
                             GParamSpec      *pspec);
 
+static gboolean flush      (MilterAgent     *agent,
+                            GError         **error);
+
 static void
 milter_agent_class_init (MilterAgentClass *klass)
 {
@@ -98,6 +112,7 @@ milter_agent_class_init (MilterAgentClass *klass)
 
     klass->decoder_new = NULL;
     klass->encoder_new = NULL;
+    klass->flush = flush;
 
     spec = g_param_spec_object("reader",
                                "Reader",
@@ -127,6 +142,29 @@ milter_agent_class_init (MilterAgentClass *klass)
                                G_PARAM_READABLE);
     g_object_class_install_property(gobject_class, PROP_ELAPSED, spec);
 
+    spec = g_param_spec_object("event-loop",
+                               "Event Loop",
+                               "The event loop of the agent",
+                               MILTER_TYPE_EVENT_LOOP,
+                               G_PARAM_READWRITE);
+    g_object_class_install_property(gobject_class, PROP_EVENT_LOOP, spec);
+
+    /**
+     * MilterAgent::flushed:
+     * @agent: the agent that received the signal.
+     *
+     * This signal is emitted when writer of the agent
+     * flushes requested data.
+     */
+    signals[FLUSHED] =
+        g_signal_new("flushed",
+                     MILTER_TYPE_AGENT,
+                     G_SIGNAL_RUN_LAST,
+                     G_STRUCT_OFFSET(MilterAgentClass, flushed),
+                     NULL, NULL,
+                     g_cclosure_marshal_VOID__VOID,
+                     G_TYPE_NONE, 0);
+
     g_type_class_add_private(gobject_class, sizeof(MilterAgentPrivate));
 }
 
@@ -188,29 +226,26 @@ milter_agent_init (MilterAgent *agent)
     priv->tag = 0;
     priv->finished = FALSE;
     priv->timer = NULL;
+    priv->event_loop = NULL;
+    priv->shutting_down = FALSE;
 }
 
 static void
 dispose (GObject *object)
 {
+    MilterAgent *agent;
     MilterAgentPrivate *priv;
 
-    priv = MILTER_AGENT_GET_PRIVATE(object);
+    agent = MILTER_AGENT(object);
+    priv = MILTER_AGENT_GET_PRIVATE(agent);
 
     if (priv->timer) {
         g_timer_destroy(priv->timer);
         priv->timer = NULL;
     }
 
-    if (priv->reader) {
-        g_object_unref(priv->reader);
-        priv->reader = NULL;
-    }
-
-    if (priv->writer) {
-        g_object_unref(priv->writer);
-        priv->writer = NULL;
-    }
+    milter_agent_set_reader(agent, NULL);
+    milter_agent_set_writer(agent, NULL);
 
     if (priv->decoder) {
         g_object_unref(priv->decoder);
@@ -222,6 +257,11 @@ dispose (GObject *object)
         priv->encoder = NULL;
     }
 
+    if (priv->event_loop) {
+        g_object_unref(priv->event_loop);
+        priv->event_loop = NULL;
+    }
+
     G_OBJECT_CLASS(milter_agent_parent_class)->dispose(object);
 }
 
@@ -231,15 +271,20 @@ set_property (GObject      *object,
               const GValue *value,
               GParamSpec   *pspec)
 {
+    MilterAgent *agent;
     MilterAgentPrivate *priv;
 
+    agent = MILTER_AGENT(object);
     priv = MILTER_AGENT_GET_PRIVATE(object);
     switch (prop_id) {
     case PROP_READER:
-        milter_agent_set_reader(MILTER_AGENT(object), g_value_get_object(value));
+        milter_agent_set_reader(agent, g_value_get_object(value));
         break;
     case PROP_TAG:
-        milter_agent_set_tag(MILTER_AGENT(object), g_value_get_uint(value));
+        milter_agent_set_tag(agent, g_value_get_uint(value));
+        break;
+    case PROP_EVENT_LOOP:
+        milter_agent_set_event_loop(agent, g_value_get_object(value));
         break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
@@ -271,12 +316,31 @@ get_property (GObject    *object,
     case PROP_ELAPSED:
         g_value_set_double(value, milter_agent_get_elapsed(agent));
         break;
+    case PROP_EVENT_LOOP:
+        g_value_set_object(value, milter_agent_get_event_loop(agent));
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
     }
 }
 
+static gboolean
+flush (MilterAgent *agent, GError **error)
+{
+    MilterAgentPrivate *priv;
+    gboolean success;
+
+    priv = MILTER_AGENT_GET_PRIVATE(agent);
+
+    if (!priv->writer)
+        return TRUE;
+
+    success = milter_writer_flush(priv->writer, error);
+
+    return success;
+}
+
 static void
 finished (MilterFinishedEmittable *emittable)
 {
@@ -365,8 +429,6 @@ milter_agent_set_reader (MilterAgent *agent, MilterReader *reader)
     priv = MILTER_AGENT_GET_PRIVATE(agent);
 
     if (priv->reader) {
-        milter_reader_set_tag(priv->reader, 0);
-
 #define DISCONNECT(name)                                                \
         g_signal_handlers_disconnect_by_func(priv->reader,              \
                                              G_CALLBACK(cb_reader_ ## name), \
@@ -402,6 +464,20 @@ milter_agent_error_quark (void)
     return g_quark_from_static_string("milter-agent-error-quark");
 }
 
+static void
+cb_writer_flushed (MilterWriter *writer, gpointer user_data)
+{
+    MilterAgent *agent = user_data;
+
+    if (milter_need_debug_log()) {
+        MilterAgentPrivate *priv;
+
+        priv = MILTER_AGENT_GET_PRIVATE(agent);
+        milter_debug("[%u] [agent][writer] flushed", priv->tag);
+    }
+    g_signal_emit(agent, signals[FLUSHED], 0);
+}
+
 static void
 cb_writer_error (MilterWriter *writer,
                  GError *writer_error,
@@ -445,8 +521,10 @@ milter_agent_write_packet (MilterAgent *agent,
     if (!priv->writer)
         return TRUE;
 
-    success = milter_writer_write(priv->writer, packet, packet_size,
-                                  NULL, error);
+    success = milter_writer_write(priv->writer, packet, packet_size, error);
+    if (success) {
+        success = milter_agent_flush(agent, error);
+    }
 
     return success;
 }
@@ -454,17 +532,10 @@ milter_agent_write_packet (MilterAgent *agent,
 gboolean
 milter_agent_flush (MilterAgent *agent, GError **error)
 {
-    MilterAgentPrivate *priv;
-    gboolean success;
+    MilterAgentClass *agent_class;
 
-    priv = MILTER_AGENT_GET_PRIVATE(agent);
-
-    if (!priv->writer)
-        return TRUE;
-
-    success = milter_writer_flush(priv->writer, error);
-
-    return success;
+    agent_class = MILTER_AGENT_GET_CLASS(agent);
+    return agent_class->flush(agent, error);
 }
 
 void
@@ -475,12 +546,20 @@ milter_agent_set_writer (MilterAgent *agent, MilterWriter *writer)
     priv = MILTER_AGENT_GET_PRIVATE(agent);
 
     if (priv->writer) {
-        milter_writer_set_tag(priv->writer, 0);
+        if (milter_writer_is_watching(priv->writer)) {
+            GError *error = NULL;
+            if (!milter_agent_flush(agent, &error)) {
+                milter_error("[%u] [agent][error][set-writer][auto-flush] %s",
+                             priv->tag, error->message);
+                g_error_free(error);
+            }
+        }
 
 #define DISCONNECT(name)                                                \
         g_signal_handlers_disconnect_by_func(priv->writer,              \
                                              G_CALLBACK(cb_writer_ ## name), \
                                              agent)
+        DISCONNECT(flushed);
         DISCONNECT(error);
         DISCONNECT(finished);
 #undef DISCONNECT
@@ -496,6 +575,7 @@ milter_agent_set_writer (MilterAgent *agent, MilterWriter *writer)
         g_signal_connect(priv->writer, #name,                           \
                          G_CALLBACK(cb_writer_ ## name),                \
                          agent)
+        CONNECT(flushed);
         CONNECT(error);
         CONNECT(finished);
 #undef CONNECT
@@ -516,13 +596,21 @@ milter_agent_get_decoder (MilterAgent *agent)
     return MILTER_AGENT_GET_PRIVATE(agent)->decoder;
 }
 
-void
-milter_agent_start (MilterAgent *agent, GMainContext *context)
+gboolean
+milter_agent_start (MilterAgent *agent, GError **error)
 {
     MilterAgentPrivate *priv;
 
     priv = MILTER_AGENT_GET_PRIVATE(agent);
 
+    if (!priv->event_loop) {
+        g_set_error(error,
+                    MILTER_AGENT_ERROR,
+                    MILTER_AGENT_ERROR_NO_EVENT_LOOP_ERROR,
+                    "agent has no event loop");
+        return FALSE;
+    }
+
     if (priv->timer) {
         g_timer_start(priv->timer);
     } else {
@@ -530,26 +618,43 @@ milter_agent_start (MilterAgent *agent, GMainContext *context)
     }
 
     if (priv->reader)
-        milter_reader_start(priv->reader, context);
+        milter_reader_start(priv->reader, priv->event_loop);
     if (priv->writer)
-        milter_writer_start(priv->writer, context);
+        milter_writer_start(priv->writer, priv->event_loop);
+
+    return TRUE;
 }
 
 void
 milter_agent_shutdown (MilterAgent *agent)
 {
     MilterAgentPrivate *priv;
+    gboolean have_reader = FALSE;
 
     priv = MILTER_AGENT_GET_PRIVATE(agent);
 
     milter_debug("[%u] [agent][shutdown]", priv->tag);
+
+    if (priv->shutting_down)
+        return;
+
+    priv->shutting_down = TRUE;
+
     if (priv->reader) {
+        have_reader = TRUE;
         milter_debug("[%u] [agent][shutdown][reader]", priv->tag);
         milter_reader_shutdown(priv->reader);
-    } else {
-        if (!priv->finished)
-            milter_finished_emittable_emit(MILTER_FINISHED_EMITTABLE(agent));
     }
+
+    if (priv->writer) {
+        milter_debug("[%u] [agent][shutdown][writer]", priv->tag);
+        milter_writer_shutdown(priv->writer);
+    }
+
+    if (!have_reader && !priv->finished) {
+        milter_finished_emittable_emit(MILTER_FINISHED_EMITTABLE(agent));
+    }
+    priv->shutting_down = FALSE;
 }
 
 guint
@@ -568,6 +673,25 @@ milter_agent_set_tag (MilterAgent *agent, guint tag)
     apply_tag(priv);
 }
 
+MilterEventLoop *
+milter_agent_get_event_loop (MilterAgent *agent)
+{
+    return MILTER_AGENT_GET_PRIVATE(agent)->event_loop;
+}
+
+void
+milter_agent_set_event_loop (MilterAgent *agent, MilterEventLoop *loop)
+{
+    MilterAgentPrivate *priv;
+
+    priv = MILTER_AGENT_GET_PRIVATE(agent);
+    if (priv->event_loop)
+        g_object_unref(priv->event_loop);
+    priv->event_loop = loop;
+    if (priv->event_loop)
+        g_object_ref(priv->event_loop);
+}
+
 gdouble
 milter_agent_get_elapsed (MilterAgent *agent)
 {
