@@ -29,7 +29,7 @@ G_BEGIN_DECLS
 
 #define milter_log(level, format, ...)                          \
     do {                                                        \
-        if (milter_get_log_level() & (level)) {                 \
+        if (milter_need_log(level)) {                           \
             (milter_logger_log(milter_logger(),                 \
                                MILTER_LOG_DOMAIN,               \
                                (level),                         \
@@ -61,9 +61,11 @@ G_BEGIN_DECLS
     milter_logger_set_target_level(milter_logger(), (level))
 #define milter_get_log_level()                          \
     milter_logger_get_resolved_target_level(milter_logger())
+#define milter_get_interesting_log_level()                      \
+    milter_logger_get_interesting_level(milter_logger())
 
 #define milter_need_log(level) \
-    (milter_get_log_level() & (level))
+    (milter_get_interesting_log_level() & (level))
 #define milter_need_critical_log() \
     (milter_need_log(MILTER_LOG_LEVEL_CRITICAL))
 #define milter_need_error_log() \
@@ -118,13 +120,17 @@ typedef enum
     MILTER_LOG_ITEM_DOMAIN      = 1 << 1,
     MILTER_LOG_ITEM_LEVEL       = 1 << 2,
     MILTER_LOG_ITEM_LOCATION    = 1 << 3,
-    MILTER_LOG_ITEM_TIME        = 1 << 4
+    MILTER_LOG_ITEM_TIME        = 1 << 4,
+    MILTER_LOG_ITEM_NAME        = 1 << 5,
+    MILTER_LOG_ITEM_PID         = 1 << 6
 } MilterLogItemFlags;
 
 #define MILTER_LOG_ITEM_ALL (MILTER_LOG_ITEM_DOMAIN |   \
                              MILTER_LOG_ITEM_LEVEL |    \
                              MILTER_LOG_ITEM_LOCATION | \
-                             MILTER_LOG_ITEM_TIME)
+                             MILTER_LOG_ITEM_TIME |     \
+                             MILTER_LOG_ITEM_NAME |     \
+                             MILTER_LOG_ITEM_PID)
 
 typedef enum
 {
@@ -202,6 +208,13 @@ void             milter_logger_set_target_level
 void             milter_logger_set_target_level_by_string
                                               (MilterLogger        *logger,
                                                const gchar         *level_name);
+void             milter_logger_set_interesting_level
+                                              (MilterLogger        *logger,
+                                               const gchar         *key,
+                                               MilterLogLevelFlags  level);
+MilterLogLevelFlags
+                 milter_logger_get_interesting_level
+                                              (MilterLogger        *logger);
 MilterLogItemFlags
                  milter_logger_get_target_item
                                               (MilterLogger        *logger);
