@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -28,6 +28,7 @@
 
 #include <milter/client.h>
 #include <milter-assertions.h>
+#include "milter-test-utils.h"
 
 #include <gcutter.h>
 
@@ -60,15 +61,14 @@ void test_feed_abort_outsize_message_processing (void);
 void test_write_error (void);
 void test_timeout (void);
 
+static MilterEventLoop *loop;
+
 static MilterClientContext *context;
 static MilterCommandEncoder *encoder;
 static MilterReplyEncoder *reply_encoder;
 static MilterWriter *writer;
 static GIOChannel *channel;
 
-static gchar *packet;
-static gsize packet_size;
-
 static gint n_negotiates;
 static gint n_negotiate_responses;
 static gint n_negotiate_progresss;
@@ -518,21 +518,25 @@ setup_signals (MilterClientContext *context)
 }
 
 void
-setup (void)
+cut_setup (void)
 {
+    GError *error = NULL;
+
+    loop = milter_test_event_loop_new();
+
     context = milter_client_context_new(NULL);
+    milter_agent_set_event_loop(MILTER_AGENT(context), loop);
     setup_signals(context);
 
     encoder = MILTER_COMMAND_ENCODER(milter_command_encoder_new());
     reply_encoder = MILTER_REPLY_ENCODER(milter_reply_encoder_new());
-    packet = NULL;
-    packet_size = 0;
 
     channel = gcut_string_io_channel_new(NULL);
     g_io_channel_set_encoding(channel, NULL, NULL);
     writer = milter_writer_io_channel_new(channel);
     milter_agent_set_writer(MILTER_AGENT(context), writer);
-    milter_agent_start(MILTER_AGENT(context), NULL);
+    milter_agent_start(MILTER_AGENT(context), &error);
+    gcut_assert_error(error);
 
     n_negotiates = 0;
     n_negotiate_responses = 0;
@@ -610,15 +614,6 @@ setup (void)
     expected_error = NULL;
 }
 
-static void
-packet_free (void)
-{
-    if (packet)
-        g_free(packet);
-    packet = NULL;
-    packet_size = 0;
-}
-
 static void
 expected_macros_free (void)
 {
@@ -646,7 +641,6 @@ expected_macro_value_free (void)
 static void
 clear_reuse_data (void)
 {
-    packet_free();
     expected_macros_free();
     macro_name_free();
     expected_macro_value_free();
@@ -654,13 +648,13 @@ clear_reuse_data (void)
 }
 
 void
-teardown (void)
+cut_teardown (void)
 {
     if (timeout_ids) {
         GList *node;
         for (node = timeout_ids; node; node = g_list_next(node)) {
             guint timeout_id = GPOINTER_TO_UINT(node->data);
-            g_source_remove(timeout_id);
+            milter_event_loop_remove(loop, timeout_id);
         }
         g_list_free(timeout_ids);
     }
@@ -678,7 +672,8 @@ teardown (void)
     if (channel)
         g_io_channel_unref(channel);
 
-    packet_free();
+    if (loop)
+        g_object_unref(loop);
 
     if (negotiate_option)
         g_object_unref(negotiate_option);
@@ -740,11 +735,13 @@ teardown (void)
 typedef void (*HookFunction) (void);
 
 static GError *
-feed (void)
+feed (const gchar *packet, gsize packet_size)
 {
     GError *error = NULL;
 
     milter_client_context_feed(context, packet, packet_size, &error);
+    if (!error)
+        milter_test_pump_all_events(loop);
 
     return error;
 }
@@ -757,6 +754,8 @@ test_feed_negotiate (void)
     guint32 version;
     MilterActionFlags action;
     MilterStepFlags step;
+    const gchar *packet;
+    gsize packet_size;
 
     version = 2;
     action = MILTER_ACTION_ADD_HEADERS |
@@ -780,7 +779,7 @@ test_feed_negotiate (void)
                                             &packet, &packet_size,
                                             option);
 
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
     cut_assert_equal_int(1, n_negotiates);
     cut_assert_equal_int(1, n_negotiate_responses);
     milter_assert_equal_state(NEGOTIATE_REPLIED);
@@ -805,6 +804,8 @@ test_feed_negotiate_progress (void)
     guint32 version;
     MilterActionFlags action;
     MilterStepFlags step;
+    const gchar *packet;
+    gsize packet_size;
 
     version = 2;
     action = MILTER_ACTION_ADD_HEADERS |
@@ -830,7 +831,7 @@ test_feed_negotiate_progress (void)
 
     g_signal_connect(context, "negotiate",
                      G_CALLBACK(cb_negotiate_progress), NULL);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
     cut_assert_equal_int(1, n_negotiate_progresss);
     cut_assert_equal_int(0, n_negotiate_responses);
 
@@ -857,6 +858,8 @@ test_feed_connect_ipv4 (void)
     const gchar ip_address[] = "192.168.123.123";
     gchar actual_address_string[INET6_ADDRSTRLEN];
     guint16 port;
+    const gchar *packet;
+    gsize packet_size;
 
     port = g_htons(50443);
     address.sin_family = AF_INET;
@@ -868,7 +871,7 @@ test_feed_connect_ipv4 (void)
                                           host_name,
                                           (const struct sockaddr *)&address,
                                           sizeof(address));
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
     cut_assert_equal_int(1, n_connects);
     milter_assert_equal_state(CONNECT_REPLIED);
 
@@ -897,6 +900,8 @@ test_feed_connect_ipv6 (void)
     const gchar ipv6_address[] = "2001:c90:625:12e8:290:ccff:fee2:80c5";
     gchar connected_ipv6_address[INET6_ADDRSTRLEN];
     guint16 port;
+    const gchar *packet;
+    gsize packet_size;
 
     port = g_htons(50443);
     address.sin6_family = AF_INET6;
@@ -908,7 +913,7 @@ test_feed_connect_ipv6 (void)
                                           host_name,
                                           (const struct sockaddr *)&address,
                                           sizeof(address));
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
     cut_assert_equal_int(1, n_connects);
     cut_assert_equal_int(1, n_connect_responses);
     milter_assert_equal_state(CONNECT_REPLIED);
@@ -937,6 +942,8 @@ test_feed_connect_unix (void)
     struct sockaddr_un *connected_address;
     const gchar host_name[] = "mx.local.net";
     const gchar path[] = "/tmp/unix.sock";
+    const gchar *packet;
+    gsize packet_size;
 
     address.sun_family = AF_UNIX;
     strcpy(address.sun_path, path);
@@ -946,7 +953,7 @@ test_feed_connect_unix (void)
                                           host_name,
                                           (const struct sockaddr *)&address,
                                           sizeof(address));
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
     cut_assert_equal_int(1, n_connects);
     cut_assert_equal_int(1, n_connect_responses);
     milter_assert_equal_state(CONNECT_REPLIED);
@@ -972,6 +979,8 @@ test_feed_connect_with_macro (void)
     const gchar ip_address[] = "192.168.123.123";
     gchar actual_address_string[INET6_ADDRSTRLEN];
     guint16 port;
+    const gchar *packet;
+    gsize packet_size;
 
     test_feed_negotiate();
     clear_reuse_data();
@@ -985,8 +994,7 @@ test_feed_connect_with_macro (void)
                                                &packet, &packet_size,
                                                MILTER_COMMAND_CONNECT,
                                                expected_macros);
-    gcut_assert_error(feed());
-    packet_free();
+    gcut_assert_error(feed(packet, packet_size));
 
     port = g_htons(50443);
     address.sin_family = AF_INET;
@@ -998,7 +1006,7 @@ test_feed_connect_with_macro (void)
                                           host_name,
                                           (const struct sockaddr *)&address,
                                           sizeof(address));
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
     cut_assert_equal_int(1, n_connects);
     cut_assert_equal_int(1, n_connect_responses);
     milter_assert_equal_state(CONNECT_REPLIED);
@@ -1026,13 +1034,15 @@ test_feed_connect_with_macro (void)
 static void
 feed_helo_pre_hook_with_macro (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     expected_macros = gcut_hash_table_string_string_new(NULL, NULL);
     milter_command_encoder_encode_define_macro(encoder,
                                                &packet, &packet_size,
                                                MILTER_COMMAND_HELO,
                                                expected_macros);
-    gcut_assert_error(feed());
-    packet_free();
+    gcut_assert_error(feed(packet, packet_size));
 
     macro_name = g_strdup("nonexistent");
 }
@@ -1049,6 +1059,8 @@ test_feed_helo (gconstpointer data)
 {
     const HookFunction pre_hook = data;
     const gchar fqdn[] = "delian";
+    const gchar *packet;
+    gsize packet_size;
 
     test_feed_connect_with_macro();
     clear_reuse_data();
@@ -1057,7 +1069,7 @@ test_feed_helo (gconstpointer data)
         pre_hook();
 
     milter_command_encoder_encode_helo(encoder, &packet, &packet_size, fqdn);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
     cut_assert_equal_int(1, n_helos);
     cut_assert_equal_int(1, n_helo_responses);
     milter_assert_equal_state(HELO_REPLIED);
@@ -1078,6 +1090,8 @@ static void
 feed_envelope_from_pre_hook_with_macro (void)
 {
     GHashTable *macros;
+    const gchar *packet;
+    gsize packet_size;
 
     macro_name = g_strdup("mail_addr");
     expected_macro_value = g_strdup("kou@example.com");
@@ -1092,8 +1106,7 @@ feed_envelope_from_pre_hook_with_macro (void)
                                        MILTER_COMMAND_ENVELOPE_FROM,
                                        macros);
     g_hash_table_unref(macros);
-    gcut_assert_error(feed());
-    packet_free();
+    gcut_assert_error(feed(packet, packet_size));
 }
 
 void
@@ -1108,6 +1121,8 @@ test_feed_envelope_from (gconstpointer data)
 {
     const HookFunction pre_hook = data;
     const gchar from[] = "<kou@example.com>";
+    const gchar *packet;
+    gsize packet_size;
 
     test_feed_helo(feed_helo_pre_hook_with_macro);
     clear_reuse_data();
@@ -1117,7 +1132,7 @@ test_feed_envelope_from (gconstpointer data)
 
     milter_command_encoder_encode_envelope_from(encoder,
                                                 &packet, &packet_size, from);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
     cut_assert_equal_int(1, n_envelope_froms);
     cut_assert_equal_int(1, n_envelope_from_responses);
     milter_assert_equal_state(ENVELOPE_FROM_REPLIED);
@@ -1138,6 +1153,8 @@ static void
 feed_envelope_recipient_pre_hook_with_macro (void)
 {
     GHashTable *macros;
+    const gchar *packet;
+    gsize packet_size;
 
     macro_name = g_strdup("{rcpt_addr}");
     expected_macro_value = g_strdup("kou@example.com");
@@ -1152,8 +1169,7 @@ feed_envelope_recipient_pre_hook_with_macro (void)
                                                MILTER_COMMAND_ENVELOPE_RECIPIENT,
                                                macros);
     g_hash_table_unref(macros);
-    gcut_assert_error(feed());
-    packet_free();
+    gcut_assert_error(feed(packet, packet_size));
 }
 
 void
@@ -1170,6 +1186,8 @@ test_feed_envelope_recipient (gconstpointer data)
 {
     const HookFunction pre_hook = data;
     const gchar to[] = "<kou@example.com>";
+    const gchar *packet;
+    gsize packet_size;
 
     test_feed_envelope_from(feed_envelope_from_pre_hook_with_macro);
     clear_reuse_data();
@@ -1179,7 +1197,7 @@ test_feed_envelope_recipient (gconstpointer data)
 
     milter_command_encoder_encode_envelope_recipient(encoder,
                                                      &packet, &packet_size, to);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
     cut_assert_equal_int(1, n_envelope_recipients);
     cut_assert_equal_int(1, n_envelope_recipient_responses);
     milter_assert_equal_state(ENVELOPE_RECIPIENT_REPLIED);
@@ -1199,11 +1217,14 @@ test_feed_envelope_recipient (gconstpointer data)
 void
 test_feed_unknown (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     test_feed_envelope_recipient(feed_envelope_recipient_pre_hook_with_macro);
     clear_reuse_data();
 
     milter_command_encoder_encode_unknown(encoder, &packet, &packet_size, "!");
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
     cut_assert_equal_int(1, n_unknowns);
     cut_assert_equal_int(1, n_unknown_responses);
     cut_assert_equal_string("!", unknown_command);
@@ -1216,6 +1237,8 @@ static void
 feed_data_pre_hook_with_macro (void)
 {
     GHashTable *macros;
+    const gchar *packet;
+    gsize packet_size;
 
     macro_name = g_strdup("{auth_type}");
     expected_macro_value = g_strdup("CRAM-MD5");
@@ -1230,8 +1253,7 @@ feed_data_pre_hook_with_macro (void)
                                                MILTER_COMMAND_DATA,
                                                macros);
     g_hash_table_unref(macros);
-    gcut_assert_error(feed());
-    packet_free();
+    gcut_assert_error(feed(packet, packet_size));
 }
 
 void
@@ -1246,6 +1268,8 @@ test_feed_data (gconstpointer data)
 {
     const HookFunction pre_hook = data;
     GString *string;
+    const gchar *packet;
+    gsize packet_size;
 
     test_feed_envelope_recipient(feed_envelope_recipient_pre_hook_with_macro);
     clear_reuse_data();
@@ -1254,12 +1278,11 @@ test_feed_data (gconstpointer data)
         pre_hook();
 
     milter_command_encoder_encode_data(encoder, &packet, &packet_size);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
     cut_assert_equal_int(1, n_datas);
     cut_assert_equal_int(1, n_data_responses);
     milter_assert_equal_state(DATA_REPLIED);
 
-    packet_free();
     milter_reply_encoder_encode_continue(reply_encoder, &packet, &packet_size);
     string = gcut_string_io_channel_get_string(channel);
     cut_assert_equal_memory(packet, packet_size, string->str, string->len);
@@ -1277,6 +1300,9 @@ test_feed_data (gconstpointer data)
 static void
 feed_header_pre_hook_with_macro (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     macro_name = g_strdup("i");
     expected_macro_value = g_strdup("69FDD42DF4A");
     expected_macros =
@@ -1285,8 +1311,7 @@ feed_header_pre_hook_with_macro (void)
                                        &packet, &packet_size,
                                        MILTER_COMMAND_HEADER,
                                        expected_macros);
-    gcut_assert_error(feed());
-    packet_free();
+    gcut_assert_error(feed(packet, packet_size));
 }
 
 void
@@ -1301,6 +1326,8 @@ test_feed_header (gconstpointer data)
 {
     const HookFunction pre_hook = data;
     const gchar date[] = "Fri,  5 Sep 2008 09:19:56 +0900 (JST)";
+    const gchar *packet;
+    gsize packet_size;
 
     test_feed_data(feed_data_pre_hook_with_macro);
     clear_reuse_data();
@@ -1310,7 +1337,7 @@ test_feed_header (gconstpointer data)
 
     milter_command_encoder_encode_header(encoder, &packet, &packet_size,
                                          "Date", date);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
     cut_assert_equal_int(1, n_headers);
     cut_assert_equal_int(1, n_header_responses);
     milter_assert_equal_state(HEADER_REPLIED);
@@ -1331,6 +1358,9 @@ test_feed_header (gconstpointer data)
 static void
 feed_end_of_header_pre_hook_with_macro (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     macro_name = g_strdup("i");
     expected_macro_value = g_strdup("69FDD42DF4A");
     expected_macros =
@@ -1339,8 +1369,7 @@ feed_end_of_header_pre_hook_with_macro (void)
                                        &packet, &packet_size,
                                        MILTER_COMMAND_END_OF_HEADER,
                                        expected_macros);
-    gcut_assert_error(feed());
-    packet_free();
+    gcut_assert_error(feed(packet, packet_size));
 }
 
 void
@@ -1354,6 +1383,8 @@ void
 test_feed_end_of_header (gconstpointer data)
 {
     const HookFunction pre_hook = data;
+    const gchar *packet;
+    gsize packet_size;
 
     test_feed_header(feed_header_pre_hook_with_macro);
     clear_reuse_data();
@@ -1362,7 +1393,7 @@ test_feed_end_of_header (gconstpointer data)
         pre_hook();
 
     milter_command_encoder_encode_end_of_header(encoder, &packet, &packet_size);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
     cut_assert_equal_int(1, n_end_of_headers);
     cut_assert_equal_int(1, n_end_of_header_responses);
     milter_assert_equal_state(END_OF_HEADER_REPLIED);
@@ -1380,6 +1411,9 @@ test_feed_end_of_header (gconstpointer data)
 static void
 feed_body_pre_hook_with_macro (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     macro_name = g_strdup("i");
     expected_macro_value = g_strdup("69FDD42DF4A");
     expected_macros =
@@ -1388,8 +1422,7 @@ feed_body_pre_hook_with_macro (void)
                                        &packet, &packet_size,
                                        MILTER_COMMAND_BODY,
                                        expected_macros);
-    gcut_assert_error(feed());
-    packet_free();
+    gcut_assert_error(feed(packet, packet_size));
 }
 
 void
@@ -1409,6 +1442,8 @@ test_feed_body (gconstpointer data)
         "La de da de da 3.\n"
         "La de da de da 4.";
     gsize packed_size;
+    const gchar *packet;
+    gsize packet_size;
 
     test_feed_end_of_header(feed_end_of_header_pre_hook_with_macro);
     clear_reuse_data();
@@ -1418,7 +1453,7 @@ test_feed_body (gconstpointer data)
 
     milter_command_encoder_encode_body(encoder, &packet, &packet_size,
                                body, sizeof(body), &packed_size);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
     cut_assert_equal_int(1, n_bodies);
     cut_assert_equal_int(1, n_body_responses);
     milter_assert_equal_state(BODY_REPLIED);
@@ -1450,6 +1485,9 @@ feed_end_of_message_pre_hook_with_chunk (void)
 static void
 feed_end_of_message_pre_hook_with_macro (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     macro_name = g_strdup("i");
     expected_macro_value = g_strdup("69FDD42DF4A");
     expected_macros =
@@ -1458,8 +1496,7 @@ feed_end_of_message_pre_hook_with_macro (void)
                                        &packet, &packet_size,
                                        MILTER_COMMAND_END_OF_MESSAGE,
                                        expected_macros);
-    gcut_assert_error(feed());
-    packet_free();
+    gcut_assert_error(feed(packet, packet_size));
 }
 
 static void
@@ -1486,6 +1523,8 @@ void
 test_feed_end_of_message (gconstpointer data)
 {
     const HookFunction pre_hook = data;
+    const gchar *packet;
+    gsize packet_size;
 
     test_feed_body(feed_body_pre_hook_with_macro);
     clear_reuse_data();
@@ -1496,7 +1535,7 @@ test_feed_end_of_message (gconstpointer data)
     milter_command_encoder_encode_end_of_message(encoder, &packet, &packet_size,
                                                  end_of_message_chunk_test_data,
                                                  end_of_message_chunk_test_data_size);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
 
     cut_assert_equal_int(1, n_bodies);
     cut_assert_equal_int(1, n_body_responses);
@@ -1525,11 +1564,14 @@ test_feed_end_of_message (gconstpointer data)
 void
 test_feed_quit (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     test_feed_end_of_message(feed_end_of_message_pre_hook_with_macro);
     clear_reuse_data();
 
     milter_command_encoder_encode_quit(encoder, &packet, &packet_size);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
     cut_assert_equal_int(1, n_finished_emissions);
     milter_assert_equal_state(FINISHED);
 
@@ -1542,11 +1584,14 @@ test_feed_quit (void)
 void
 test_feed_abort (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     test_feed_body(feed_body_pre_hook_with_macro);
     clear_reuse_data();
 
     milter_command_encoder_encode_abort(encoder, &packet, &packet_size);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
     cut_assert_equal_int(1, n_aborts);
     cut_assert_equal_int(1, n_abort_responses);
     milter_assert_equal_state(ABORT_REPLIED);
@@ -1564,11 +1609,14 @@ test_feed_abort (void)
 void
 test_feed_abort_outsize_message_processing (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     test_feed_helo(feed_helo_pre_hook_with_macro);
     clear_reuse_data();
 
     milter_command_encoder_encode_abort(encoder, &packet, &packet_size);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
     cut_assert_equal_int(1, n_aborts);
     cut_assert_equal_int(1, n_abort_responses);
     milter_assert_equal_state(ABORT_REPLIED);
@@ -1612,15 +1660,19 @@ test_timeout (void)
     cut_assert_equal_uint(1, milter_client_context_get_timeout(context));
 
     g_signal_connect(context, "timeout", G_CALLBACK(cb_timeout), &timed_out);
-    timeout_id = g_timeout_add(500, cb_timeout_detect, &timed_out_before);
+    timeout_id = milter_event_loop_add_timeout(loop, 0.5,
+                                               cb_timeout_detect,
+                                               &timed_out_before);
     timeout_ids = g_list_append(timeout_ids, GUINT_TO_POINTER(timeout_id));
-    timeout_id = g_timeout_add(2000, cb_timeout_detect, &timed_out_after);
+    timeout_id = milter_event_loop_add_timeout(loop, 2,
+                                               cb_timeout_detect,
+                                               &timed_out_after);
     timeout_ids = g_list_append(timeout_ids, GUINT_TO_POINTER(timeout_id));
 
     milter_client_context_progress(context);
 
     while (!timed_out && !timed_out_after)
-        g_main_context_iteration(NULL, TRUE);
+        milter_event_loop_iterate(loop, TRUE);
     cut_assert_true(timed_out_before);
     cut_assert_true(timed_out);
     cut_assert_false(timed_out_after);
@@ -1636,14 +1688,20 @@ test_write_error (void)
     gcut_assert_error(error);
     gcut_string_io_channel_set_limit(channel, 1);
 
-    expected_error = g_error_new(MILTER_CLIENT_CONTEXT_ERROR,
-                                 MILTER_CLIENT_CONTEXT_ERROR_IO_ERROR,
-                                 "Failed to write to MTA: "
-                                 "g-io-channel-error-quark:4: %s",
-                                 g_strerror(ENOSPC));
-
     milter_client_context_progress(context);
+    milter_test_pump_all_events(loop);
+
     cut_assert_equal_int(1, n_errors);
+    expected_error = g_error_new(MILTER_AGENT_ERROR,
+                                 MILTER_AGENT_ERROR_IO_ERROR,
+                                 "Output error: "
+                                 "%s:%d: failed to write: "
+                                 "%s:%d: %s",
+                                 g_quark_to_string(MILTER_WRITER_ERROR),
+                                 MILTER_WRITER_ERROR_IO_ERROR,
+                                 g_quark_to_string(G_IO_CHANNEL_ERROR),
+                                 g_io_channel_error_from_errno(ENOSPC),
+                                 g_strerror(ENOSPC));
     gcut_assert_equal_error(expected_error, actual_error);
 }
 
