@@ -64,6 +64,7 @@ void test_need_maintain_no_processing_sessions_no_interval (void);
 void test_n_workers (void);
 void test_custom_fork (void);
 void test_default_packet_buffer_size (void);
+void test_worker_id (void);
 
 static MilterEventLoop *loop;
 
@@ -1019,6 +1020,13 @@ test_default_packet_buffer_size (void)
                           milter_client_context_get_packet_buffer_size(context));
 }
 
+void
+test_worker_id (void)
+{
+    cut_assert_equal_uint(0, milter_client_get_worker_id(client));
+}
+
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
