@@ -110,10 +110,11 @@ milter_test_get_tmp_dir (void)
 
     dir = g_getenv("TMPDIR");
     if (!dir)
-        dir = "/tmp";
+        dir = milter_test_get_build_dir();
 
-    tmp_dir = g_build_filename(dir, "milter-test-XXXXXX", NULL);
-    if (!mkdtemp(tmp_dir)) {
+    tmp_dir = g_build_filename(dir, "milter-test", NULL);
+    cut_make_directory(tmp_dir, NULL);
+    if (!g_file_test(tmp_dir, G_FILE_TEST_IS_DIR)) {
         g_free(tmp_dir);
         return NULL;
     }
