@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -48,9 +48,14 @@ void test_custom_configuration_directory (void);
 void test_controller_connection_spec (void);
 void test_manager_connection_spec (void);
 void test_fallback_status (void);
+void test_fallback_status_at_disconnect (void);
 void test_package_platform (void);
 void test_package_options (void);
+void test_event_loop_backend (void);
 void test_connection_check_interval (void);
+void test_location (void);
+void test_n_workers (void);
+void test_default_packet_buffer_size (void);
 void test_egg (void);
 void test_find_egg (void);
 void test_remove_egg (void);
@@ -63,9 +68,9 @@ void test_load_absolute_path (void);
 void test_save_custom (void);
 void test_to_xml_full (void);
 void test_to_xml_signal (void);
-void test_location (void);
 
 static MilterManagerConfiguration *config;
+static MilterEventLoop *loop;
 static MilterManagerEgg *egg;
 static MilterManagerEgg *another_egg;
 static MilterManagerApplicableCondition *condition;
@@ -83,14 +88,16 @@ static gchar *actual_xml;
 static gchar *tmp_dir;
 
 void
-setup (void)
+cut_setup (void)
 {
     config = milter_manager_configuration_new(NULL);
+    loop = milter_test_event_loop_new();
+
     egg = NULL;
     another_egg = NULL;
     condition = NULL;
 
-    expected_children = milter_manager_children_new(config);
+    expected_children = milter_manager_children_new(config, loop);
     actual_children = NULL;
 
     expected_load_paths = NULL;
@@ -109,10 +116,12 @@ setup (void)
 }
 
 void
-teardown (void)
+cut_teardown (void)
 {
     if (config)
         g_object_unref(config);
+    if (loop)
+        g_object_unref(loop);
     if (egg)
         g_object_unref(egg);
     if (another_egg)
@@ -201,7 +210,7 @@ test_children (void)
     milter_manager_children_add_child(expected_children, child);
 
     cut_assert_false(attached_to);
-    actual_children = milter_manager_children_new(config);
+    actual_children = milter_manager_children_new(config, loop);
     milter_manager_configuration_setup_children(config, actual_children, NULL);
     milter_assert_equal_children(expected_children, actual_children);
     cut_assert_true(attached_to);
@@ -453,6 +462,25 @@ test_fallback_status (void)
                            MILTER_STATUS_REJECT, actual_status);
 }
 
+void
+test_fallback_status_at_disconnect (void)
+{
+    MilterStatus actual_status;
+
+    actual_status =
+        milter_manager_configuration_get_fallback_status_at_disconnect(config);
+    gcut_assert_equal_enum(MILTER_TYPE_STATUS,
+                           MILTER_STATUS_TEMPORARY_FAILURE, actual_status);
+
+    milter_manager_configuration_set_fallback_status_at_disconnect(
+        config, MILTER_STATUS_REJECT);
+
+    actual_status =
+        milter_manager_configuration_get_fallback_status_at_disconnect(config);
+    gcut_assert_equal_enum(MILTER_TYPE_STATUS,
+                           MILTER_STATUS_REJECT, actual_status);
+}
+
 void
 test_package_platform (void)
 {
@@ -489,6 +517,24 @@ test_package_options (void)
                             actual_package_options);
 }
 
+void
+test_event_loop_backend (void)
+{
+    MilterClientEventLoopBackend backend;
+
+    gcut_assert_equal_enum(
+        MILTER_TYPE_CLIENT_EVENT_LOOP_BACKEND,
+        MILTER_CLIENT_EVENT_LOOP_BACKEND_GLIB,
+        milter_manager_configuration_get_event_loop_backend(config));
+
+    backend = MILTER_CLIENT_EVENT_LOOP_BACKEND_LIBEV;
+    milter_manager_configuration_set_event_loop_backend(config, backend);
+    gcut_assert_equal_enum(
+        MILTER_TYPE_CLIENT_EVENT_LOOP_BACKEND,
+        backend,
+        milter_manager_configuration_get_event_loop_backend(config));
+}
+
 void
 test_connection_check_interval (void)
 {
@@ -501,6 +547,30 @@ test_connection_check_interval (void)
         milter_manager_configuration_get_connection_check_interval(config));
 }
 
+void
+test_n_workers (void)
+{
+    cut_assert_equal_uint(
+        0,
+        milter_manager_configuration_get_n_workers(config));
+    milter_manager_configuration_set_n_workers(config, 5);
+    cut_assert_equal_uint(
+        5,
+        milter_manager_configuration_get_n_workers(config));
+}
+
+void
+test_default_packet_buffer_size (void)
+{
+    cut_assert_equal_uint(
+        0,
+        milter_manager_configuration_get_default_packet_buffer_size(config));
+    milter_manager_configuration_set_default_packet_buffer_size(config, 4096);
+    cut_assert_equal_uint(
+        4096,
+        milter_manager_configuration_get_default_packet_buffer_size(config));
+}
+
 static void
 milter_assert_default_configuration_helper (MilterManagerConfiguration *config)
 {
@@ -573,12 +643,24 @@ milter_assert_default_configuration_helper (MilterManagerConfiguration *config)
         0,
         milter_manager_configuration_get_connection_check_interval(config));
 
+    gcut_assert_equal_enum(
+        MILTER_TYPE_CLIENT_EVENT_LOOP_BACKEND,
+        MILTER_CLIENT_EVENT_LOOP_BACKEND_GLIB,
+        milter_manager_configuration_get_event_loop_backend(config));
+    cut_assert_equal_uint(
+        0,
+        milter_manager_configuration_get_n_workers(config));
+
+    cut_assert_equal_uint(
+        0,
+        milter_manager_configuration_get_default_packet_buffer_size(config));
+
     if (expected_children)
         g_object_unref(expected_children);
-    expected_children = milter_manager_children_new(config);
+    expected_children = milter_manager_children_new(config, loop);
     if (actual_children)
         g_object_unref(actual_children);
-    actual_children = milter_manager_children_new(config);
+    actual_children = milter_manager_children_new(config, loop);
     milter_manager_configuration_setup_children(config, actual_children, NULL);
     milter_assert_equal_children(expected_children, actual_children);
     g_object_unref(actual_children);
@@ -789,6 +871,8 @@ test_clear (void)
     test_max_connections();
     test_max_file_descriptors();
     test_connection_check_interval();
+    test_n_workers();
+    test_default_packet_buffer_size();
 
     handler_id = g_signal_connect(config, "connected",
                                   G_CALLBACK(cb_connected), NULL);
