@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -33,25 +33,37 @@
 void test_negotiate_reply (void);
 void test_continue (void);
 void test_reply_code_temporary_failure (void);
+void test_reply_code_temporary_failure_on_invalid_state (void);
 void test_reply_code_reject (void);
+void test_reply_code_reject_on_invalid_state (void);
 void test_temporary_failure (void);
 void test_reject (void);
 void test_accept (void);
 void test_discard (void);
 void test_add_header (void);
+void test_add_header_on_invalid_state (void);
 void test_insert_header (void);
+void test_insert_header_on_invalid_state (void);
 void test_change_header (void);
+void test_change_header_on_invalid_state (void);
 void test_delete_header (void);
+void test_delete_header_on_invalid_state (void);
 void test_change_from (void);
+void test_change_from_on_invalid_state (void);
 void test_add_recipient (void);
+void test_add_recipient_on_invalid_state (void);
 void test_delete_recipient (void);
+void test_delete_recipient_on_invalid_state (void);
 void test_replace_body (void);
+void test_replace_body_on_invalid_state (void);
 void test_progress (void);
+void test_progress_on_invalid_state (void);
 void test_quarantine (void);
+void test_quarantine_on_invalid_state (void);
 void test_connection_failure (void);
 void test_shutdown (void);
 void test_skip (void);
-void test_skip_in_invalid_state (void);
+void test_skip_on_invalid_state (void);
 void test_reading_timeout (void);
 void test_writing_timeout (void);
 void test_invalid_state_error (void);
@@ -60,19 +72,19 @@ void test_no_busy_on_abort (void);
 void test_decode_error (void);
 void test_write_error (void);
 
+static MilterEventLoop *loop;
 static MilterServerContext *context;
 
 static MilterReader *reader;
 static MilterWriter *writer;
 
 static MilterReplyEncoder *encoder;
-static gchar *packet;
-static gsize packet_size;
 
 static GError *actual_error;
 static GError *expected_error;
 
-static GIOChannel *channel;
+static GIOChannel *read_channel;
+static GIOChannel *write_channel;
 static MilterOption *option;
 static MilterMacrosRequests *requests;
 
@@ -351,8 +363,13 @@ cb_skip (MilterServerContext *context, gpointer user_data)
 static void
 cb_error (MilterErrorEmittable *emittable, GError *error)
 {
+    GError *local_error;
+
     n_errors++;
 
+    local_error = actual_error;
+    actual_error = NULL;
+    gcut_assert_error(local_error);
     actual_error = g_error_copy(error);
 }
 
@@ -390,9 +407,14 @@ setup_signals (MilterServerContext *context)
 }
 
 void
-setup (void)
+cut_setup (void)
 {
+    GError *error = NULL;
+
+    loop = milter_test_event_loop_new();
+
     context = milter_server_context_new();
+    milter_agent_set_event_loop(MILTER_AGENT(context), loop);
     setup_signals(context);
 
     {
@@ -412,19 +434,22 @@ setup (void)
         g_object_unref(option);
     }
 
-    channel = gcut_string_io_channel_new(NULL);
-    g_io_channel_set_encoding(channel, NULL, NULL);
-    reader = milter_reader_io_channel_new(channel);
+    read_channel = gcut_string_io_channel_new(NULL);
+    gcut_string_io_channel_set_pipe_mode(read_channel, TRUE);
+    g_io_channel_set_encoding(read_channel, NULL, NULL);
+    g_io_channel_set_buffered(read_channel, FALSE);
+    reader = milter_reader_io_channel_new(read_channel);
     milter_agent_set_reader(MILTER_AGENT(context), reader);
 
-    writer = milter_writer_io_channel_new(channel);
+    write_channel = gcut_string_io_channel_new(NULL);
+    g_io_channel_set_encoding(write_channel, NULL, NULL);
+    writer = milter_writer_io_channel_new(write_channel);
     milter_agent_set_writer(MILTER_AGENT(context), writer);
 
-    milter_agent_start(MILTER_AGENT(context), NULL);
+    milter_agent_start(MILTER_AGENT(context), &error);
+    gcut_assert_error(error);
 
     encoder = MILTER_REPLY_ENCODER(milter_reply_encoder_new());
-    packet = NULL;
-    packet_size = 0;
 
     option = NULL;
     requests = NULL;
@@ -477,20 +502,11 @@ setup (void)
     buffer = g_string_new(NULL);
 }
 
-static void
-packet_free (void)
-{
-    if (packet)
-        g_free(packet);
-    packet = NULL;
-    packet_size = 0;
-}
-
 void
-teardown (void)
+cut_teardown (void)
 {
     if (timeout_id > 0)
-        g_source_remove(timeout_id);
+        milter_event_loop_remove(loop, timeout_id);
 
     if (context)
         g_object_unref(context);
@@ -504,9 +520,14 @@ teardown (void)
     if (encoder)
         g_object_unref(encoder);
 
-    if (channel) {
-        gcut_string_io_channel_set_limit(channel, 0);
-        g_io_channel_unref(channel);
+    if (read_channel) {
+        gcut_string_io_channel_set_limit(read_channel, 0);
+        g_io_channel_unref(read_channel);
+    }
+
+    if (write_channel) {
+        gcut_string_io_channel_set_limit(write_channel, 0);
+        g_io_channel_unref(write_channel);
     }
 
     if (option)
@@ -514,8 +535,6 @@ teardown (void)
     if (requests)
         g_object_unref(requests);
 
-    packet_free();
-
     if (actual_error)
         g_error_free(actual_error);
     if (expected_error)
@@ -548,43 +567,45 @@ teardown (void)
 
     if (buffer)
         g_string_free(buffer, TRUE);
-}
 
-static gboolean
-cb_idle_func (gpointer data)
-{
-    data_written = TRUE;
-    return FALSE;
+    if (loop)
+        g_object_unref(loop);
 }
 
 static void
-wait_for_written (void)
+pump_all_events (void)
 {
-    data_written = FALSE;
-    g_idle_add_full(G_PRIORITY_LOW, cb_idle_func, NULL, NULL);
+    GError *error;
 
-    while (!data_written) {
-        g_main_context_iteration(NULL, TRUE);
-    }
+    milter_test_pump_all_events(loop);
+    error = actual_error;
+    actual_error = NULL;
+    gcut_assert_error(error);
+}
+
+static void
+write_data_without_error_handling (const gchar *data, gsize data_size)
+{
+    gsize bytes_written;
+    GError *error = NULL;
+
+    g_io_channel_write_chars(read_channel, data, data_size,
+                             &bytes_written, &error);
+    gcut_assert_error(error);
+    cut_assert_equal_int(data_size, bytes_written);
+
+    milter_test_pump_all_events(loop);
 }
 
 static void
 write_data (const gchar *data, gsize data_size)
 {
-    gsize bytes_written;
-    gint64 read_offset;
-    GError *error = NULL;
+    GError *error;
 
-    g_io_channel_write_chars(channel, data, data_size,
-                             &bytes_written, &error);
+    write_data_without_error_handling(data, data_size);
+    error = actual_error;
+    actual_error = NULL;
     gcut_assert_error(error);
-    cut_assert_equal_int(data_size, bytes_written);
-
-    read_offset = -(gint64)bytes_written;
-    g_io_channel_seek_position(channel, read_offset, G_SEEK_CUR, &error);
-    gcut_assert_error(error);
-
-    wait_for_written();
 }
 
 #define milter_assert_status(expected)                                  \
@@ -595,6 +616,9 @@ write_data (const gchar *data, gsize data_size)
 void
 test_negotiate_reply (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     option = milter_option_new(2,
                                MILTER_ACTION_ADD_HEADERS,
                                MILTER_STEP_NO_HEADERS);
@@ -604,6 +628,8 @@ test_negotiate_reply (void)
                                        "G", "N", "U", NULL);
 
     milter_server_context_negotiate(context, option);
+    pump_all_events();
+
     milter_reply_encoder_encode_negotiate(encoder,
                                           &packet, &packet_size,
                                           option, requests);
@@ -617,7 +643,11 @@ test_negotiate_reply (void)
 void
 test_continue (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     milter_server_context_helo(context, "delian");
+    pump_all_events();
 
     milter_reply_encoder_encode_continue(encoder, &packet, &packet_size);
     write_data(packet, packet_size);
@@ -628,8 +658,13 @@ test_continue (void)
 void
 test_reply_code_temporary_failure (void)
 {
+    const gchar *packet;
+    gsize packet_size;
     const gchar code[] = "451 4.7.1 Greylisting in action, please come back";
 
+    milter_server_context_set_state(context,
+                                    MILTER_SERVER_CONTEXT_STATE_DATA);
+
     milter_reply_encoder_encode_reply_code(encoder, &packet, &packet_size, code);
     write_data(packet, packet_size);
     cut_assert_equal_int(1, n_reply_codes);
@@ -640,11 +675,38 @@ test_reply_code_temporary_failure (void)
     milter_assert_status(MILTER_STATUS_TEMPORARY_FAILURE);
 }
 
+void
+test_reply_code_temporary_failure_on_invalid_state (void)
+{
+    const gchar *packet;
+    gsize packet_size;
+    const gchar code[] = "451 4.7.1 Greylisting in action, please come back";
+
+    milter_reply_encoder_encode_reply_code(encoder, &packet, &packet_size, code);
+    write_data_without_error_handling(packet, packet_size);
+    cut_assert_equal_int(0, n_reply_codes);
+    cut_assert_equal_int(1, n_errors);
+
+    expected_error = g_error_new(MILTER_SERVER_CONTEXT_ERROR,
+                                 MILTER_SERVER_CONTEXT_ERROR_INVALID_STATE,
+                                 "<reply-code> should be caused on "
+                                 "<connect, helo, envelope-from, "
+                                 "envelope-recipient, data, header, "
+                                 "end-of-header, body, end-of-message, "
+                                 "unknown> but was <start>");
+    gcut_assert_equal_error(expected_error, actual_error);
+}
+
 void
 test_reply_code_reject (void)
 {
+    const gchar *packet;
+    gsize packet_size;
     const gchar code[] = "554 5.7.1 1% 2%% 3%%%";
 
+    milter_server_context_set_state(context,
+                                    MILTER_SERVER_CONTEXT_STATE_DATA);
+
     milter_reply_encoder_encode_reply_code(encoder, &packet, &packet_size, code);
     write_data(packet, packet_size);
     cut_assert_equal_int(1, n_reply_codes);
@@ -655,9 +717,34 @@ test_reply_code_reject (void)
     milter_assert_status(MILTER_STATUS_REJECT);
 }
 
+void
+test_reply_code_reject_on_invalid_state (void)
+{
+    const gchar *packet;
+    gsize packet_size;
+    const gchar code[] = "554 5.7.1 1% 2%% 3%%%";
+
+    milter_reply_encoder_encode_reply_code(encoder, &packet, &packet_size, code);
+    write_data_without_error_handling(packet, packet_size);
+    cut_assert_equal_int(0, n_reply_codes);
+    cut_assert_equal_int(1, n_errors);
+
+    expected_error = g_error_new(MILTER_SERVER_CONTEXT_ERROR,
+                                 MILTER_SERVER_CONTEXT_ERROR_INVALID_STATE,
+                                 "<reply-code> should be caused on "
+                                 "<connect, helo, envelope-from, "
+                                 "envelope-recipient, data, header, "
+                                 "end-of-header, body, end-of-message, "
+                                 "unknown> but was <start>");
+    gcut_assert_equal_error(expected_error, actual_error);
+}
+
 void
 test_temporary_failure (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     milter_reply_encoder_encode_temporary_failure(encoder, &packet, &packet_size);
     write_data(packet, packet_size);
     cut_assert_equal_int(1, n_temporary_failures);
@@ -667,6 +754,9 @@ test_temporary_failure (void)
 void
 test_reject (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     milter_reply_encoder_encode_reject(encoder, &packet, &packet_size);
     write_data(packet, packet_size);
     cut_assert_equal_int(1, n_rejects);
@@ -676,6 +766,9 @@ test_reject (void)
 void
 test_accept (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     milter_reply_encoder_encode_accept(encoder, &packet, &packet_size);
     write_data(packet, packet_size);
     cut_assert_equal_int(1, n_accepts);
@@ -685,6 +778,9 @@ test_accept (void)
 void
 test_discard (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     milter_reply_encoder_encode_discard(encoder, &packet, &packet_size);
     write_data(packet, packet_size);
     cut_assert_equal_int(1, n_discards);
@@ -694,11 +790,15 @@ test_discard (void)
 void
 test_add_header (void)
 {
+    const gchar *packet;
+    gsize packet_size;
     const gchar name[] = "X-HEADER-NAME";
     const gchar value[] = "MilterServerContext test";
 
     milter_server_context_end_of_message(context, NULL, 0);
+    pump_all_events();
 
+    cut_assert_true(milter_server_context_is_processing(context));
     milter_reply_encoder_encode_add_header(encoder,
                                            &packet, &packet_size,
                                            name, value);
@@ -707,16 +807,43 @@ test_add_header (void)
     cut_assert_equal_string(name, actual_header_name);
     cut_assert_equal_string(value, actual_header_value);
     milter_assert_status(MILTER_STATUS_NOT_CHANGE);
+    cut_assert_true(milter_server_context_is_processing(context));
+}
+
+void
+test_add_header_on_invalid_state (void)
+{
+    const gchar *packet;
+    gsize packet_size;
+    const gchar name[] = "X-HEADER-NAME";
+    const gchar value[] = "MilterServerContext test";
+
+    milter_reply_encoder_encode_add_header(encoder,
+                                           &packet, &packet_size,
+                                           name, value);
+    write_data_without_error_handling(packet, packet_size);
+    cut_assert_equal_int(0, n_add_headers);
+    cut_assert_equal_int(1, n_errors);
+
+    expected_error = g_error_new(MILTER_SERVER_CONTEXT_ERROR,
+                                 MILTER_SERVER_CONTEXT_ERROR_INVALID_STATE,
+                                 "<add-header> should be caused on "
+                                 "<end-of-message> but was <start>");
+    gcut_assert_equal_error(expected_error, actual_error);
 }
 
 void
 test_insert_header (void)
 {
+    const gchar *packet;
+    gsize packet_size;
     const gchar name[] = "X-HEADER-NAME";
     const gchar value[] = "MilterServerContext test";
 
     milter_server_context_end_of_message(context, NULL, 0);
+    pump_all_events();
 
+    cut_assert_true(milter_server_context_is_processing(context));
     milter_reply_encoder_encode_insert_header(encoder,
                                               &packet, &packet_size,
                                               G_MAXUINT32, name, value);
@@ -726,16 +853,43 @@ test_insert_header (void)
     cut_assert_equal_string(name, actual_header_name);
     cut_assert_equal_string(value, actual_header_value);
     milter_assert_status(MILTER_STATUS_NOT_CHANGE);
+    cut_assert_true(milter_server_context_is_processing(context));
+}
+
+void
+test_insert_header_on_invalid_state (void)
+{
+    const gchar *packet;
+    gsize packet_size;
+    const gchar name[] = "X-HEADER-NAME";
+    const gchar value[] = "MilterServerContext test";
+
+    milter_reply_encoder_encode_insert_header(encoder,
+                                              &packet, &packet_size,
+                                              G_MAXUINT32, name, value);
+    write_data_without_error_handling(packet, packet_size);
+    cut_assert_equal_int(0, n_insert_headers);
+    cut_assert_equal_int(1, n_errors);
+
+    expected_error = g_error_new(MILTER_SERVER_CONTEXT_ERROR,
+                                 MILTER_SERVER_CONTEXT_ERROR_INVALID_STATE,
+                                 "<insert-header> should be caused on "
+                                 "<end-of-message> but was <start>");
+    gcut_assert_equal_error(expected_error, actual_error);
 }
 
 void
 test_change_header (void)
 {
+    const gchar *packet;
+    gsize packet_size;
     const gchar name[] = "X-HEADER-NAME";
     const gchar value[] = "MilterServerContext test";
 
     milter_server_context_end_of_message(context, NULL, 0);
+    pump_all_events();
 
+    cut_assert_true(milter_server_context_is_processing(context));
     milter_reply_encoder_encode_change_header(encoder, &packet, &packet_size,
                                               name, 0, value);
     write_data(packet, packet_size);
@@ -744,15 +898,41 @@ test_change_header (void)
     cut_assert_equal_string(name, actual_header_name);
     cut_assert_equal_string(value, actual_header_value);
     milter_assert_status(MILTER_STATUS_NOT_CHANGE);
+    cut_assert_true(milter_server_context_is_processing(context));
+}
+
+void
+test_change_header_on_invalid_state (void)
+{
+    const gchar *packet;
+    gsize packet_size;
+    const gchar name[] = "X-HEADER-NAME";
+    const gchar value[] = "MilterServerContext test";
+
+    milter_reply_encoder_encode_change_header(encoder, &packet, &packet_size,
+                                              name, 0, value);
+    write_data_without_error_handling(packet, packet_size);
+    cut_assert_equal_int(0, n_change_headers);
+    cut_assert_equal_int(1, n_errors);
+
+    expected_error = g_error_new(MILTER_SERVER_CONTEXT_ERROR,
+                                 MILTER_SERVER_CONTEXT_ERROR_INVALID_STATE,
+                                 "<change-header> should be caused on "
+                                 "<end-of-message> but was <start>");
+    gcut_assert_equal_error(expected_error, actual_error);
 }
 
 void
 test_delete_header (void)
 {
+    const gchar *packet;
+    gsize packet_size;
     const gchar name[] = "X-HEADER-NAME";
 
     milter_server_context_end_of_message(context, NULL, 0);
+    pump_all_events();
 
+    cut_assert_true(milter_server_context_is_processing(context));
     milter_reply_encoder_encode_delete_header(encoder, &packet, &packet_size,
                                               name, 0);
     write_data(packet, packet_size);
@@ -760,14 +940,40 @@ test_delete_header (void)
     cut_assert_equal_int(0, actual_header_index);
     cut_assert_equal_string(name, actual_header_name);
     milter_assert_status(MILTER_STATUS_NOT_CHANGE);
+    cut_assert_true(milter_server_context_is_processing(context));
+}
+
+void
+test_delete_header_on_invalid_state (void)
+{
+    const gchar *packet;
+    gsize packet_size;
+    const gchar name[] = "X-HEADER-NAME";
+
+    milter_reply_encoder_encode_delete_header(encoder, &packet, &packet_size,
+                                              name, 0);
+    write_data_without_error_handling(packet, packet_size);
+    cut_assert_equal_int(0, n_delete_headers);
+    cut_assert_equal_int(1, n_errors);
+
+    expected_error = g_error_new(MILTER_SERVER_CONTEXT_ERROR,
+                                 MILTER_SERVER_CONTEXT_ERROR_INVALID_STATE,
+                                 "<delete-header> should be caused on "
+                                 "<end-of-message> but was <start>");
+    gcut_assert_equal_error(expected_error, actual_error);
 }
 
 void
 test_change_from (void)
 {
+    const gchar *packet;
+    gsize packet_size;
     const gchar from[] = "example@example.com";
 
     milter_server_context_end_of_message(context, NULL, 0);
+    pump_all_events();
+
+    cut_assert_true(milter_server_context_is_processing(context));
     milter_reply_encoder_encode_change_from(encoder,
                                             &packet, &packet_size,
                                             from, NULL);
@@ -775,15 +981,41 @@ test_change_from (void)
     cut_assert_equal_int(1, n_change_froms);
     cut_assert_equal_string(from, actual_envelope_from);
     milter_assert_status(MILTER_STATUS_NOT_CHANGE);
+    cut_assert_true(milter_server_context_is_processing(context));
+}
+
+void
+test_change_from_on_invalid_state (void)
+{
+    const gchar *packet;
+    gsize packet_size;
+    const gchar from[] = "example@example.com";
+
+    milter_reply_encoder_encode_change_from(encoder,
+                                            &packet, &packet_size,
+                                            from, NULL);
+    write_data_without_error_handling(packet, packet_size);
+    cut_assert_equal_int(0, n_change_froms);
+    cut_assert_equal_int(1, n_errors);
+
+    expected_error = g_error_new(MILTER_SERVER_CONTEXT_ERROR,
+                                 MILTER_SERVER_CONTEXT_ERROR_INVALID_STATE,
+                                 "<change-from> should be caused on "
+                                 "<end-of-message> but was <start>");
+    gcut_assert_equal_error(expected_error, actual_error);
 }
 
 void
 test_add_recipient (void)
 {
+    const gchar *packet;
+    gsize packet_size;
     const gchar recipient[] = "example@example.com";
 
     milter_server_context_end_of_message(context, NULL, 0);
+    pump_all_events();
 
+    cut_assert_true(milter_server_context_is_processing(context));
     milter_reply_encoder_encode_add_recipient(encoder,
                                               &packet, &packet_size,
                                               recipient, NULL);
@@ -792,13 +1024,41 @@ test_add_recipient (void)
     cut_assert_equal_string(recipient, actual_recipient);
     cut_assert_null(actual_recipient_parameters);
     milter_assert_status(MILTER_STATUS_NOT_CHANGE);
+    cut_assert_true(milter_server_context_is_processing(context));
+}
+
+void
+test_add_recipient_on_invalid_state (void)
+{
+    const gchar *packet;
+    gsize packet_size;
+    const gchar recipient[] = "example@example.com";
+
+    milter_reply_encoder_encode_add_recipient(encoder,
+                                              &packet, &packet_size,
+                                              recipient, NULL);
+    write_data_without_error_handling(packet, packet_size);
+    cut_assert_equal_int(0, n_add_recipients);
+    cut_assert_equal_int(1, n_errors);
+
+    expected_error = g_error_new(MILTER_SERVER_CONTEXT_ERROR,
+                                 MILTER_SERVER_CONTEXT_ERROR_INVALID_STATE,
+                                 "<add-recipient> should be caused on "
+                                 "<end-of-message> but was <start>");
+    gcut_assert_equal_error(expected_error, actual_error);
 }
 
 void
 test_delete_recipient (void)
 {
+    const gchar *packet;
+    gsize packet_size;
     const gchar recipient[] = "example@example.com";
 
+    milter_server_context_end_of_message(context, NULL, 0);
+    pump_all_events();
+
+    cut_assert_true(milter_server_context_is_processing(context));
     milter_reply_encoder_encode_delete_recipient(encoder,
                                                  &packet, &packet_size,
                                                  recipient);
@@ -806,47 +1066,150 @@ test_delete_recipient (void)
     cut_assert_equal_int(1, n_delete_recipients);
     cut_assert_equal_string(recipient, actual_recipient);
     milter_assert_status(MILTER_STATUS_NOT_CHANGE);
+    cut_assert_true(milter_server_context_is_processing(context));
+}
+
+void
+test_delete_recipient_on_invalid_state (void)
+{
+    const gchar *packet;
+    gsize packet_size;
+    const gchar recipient[] = "example@example.com";
+
+    milter_reply_encoder_encode_delete_recipient(encoder,
+                                                 &packet, &packet_size,
+                                                 recipient);
+    write_data_without_error_handling(packet, packet_size);
+    cut_assert_equal_int(0, n_delete_recipients);
+    cut_assert_equal_int(1, n_errors);
+
+    expected_error = g_error_new(MILTER_SERVER_CONTEXT_ERROR,
+                                 MILTER_SERVER_CONTEXT_ERROR_INVALID_STATE,
+                                 "<delete-recipient> should be caused on "
+                                 "<end-of-message> but was <start>");
+    gcut_assert_equal_error(expected_error, actual_error);
 }
 
 void
 test_replace_body (void)
 {
+    const gchar *packet;
+    gsize packet_size;
     const gchar chunk[] = "This is a body text.";
     gsize packed_size;
 
     milter_server_context_end_of_message(context, NULL, 0);
+    pump_all_events();
 
+    cut_assert_true(milter_server_context_is_processing(context));
     milter_reply_encoder_encode_replace_body(encoder, &packet, &packet_size,
                                              chunk, strlen(chunk), &packed_size);
     write_data(packet, packet_size);
     cut_assert_equal_int(1, n_replace_bodys);
     milter_assert_status(MILTER_STATUS_NOT_CHANGE);
+    cut_assert_true(milter_server_context_is_processing(context));
+}
+
+void
+test_replace_body_on_invalid_state (void)
+{
+    const gchar *packet;
+    gsize packet_size;
+    const gchar chunk[] = "This is a body text.";
+    gsize packed_size;
+
+    milter_reply_encoder_encode_replace_body(encoder, &packet, &packet_size,
+                                             chunk, strlen(chunk), &packed_size);
+    write_data_without_error_handling(packet, packet_size);
+    cut_assert_equal_int(0, n_replace_bodys);
+    cut_assert_equal_int(1, n_errors);
+
+    expected_error = g_error_new(MILTER_SERVER_CONTEXT_ERROR,
+                                 MILTER_SERVER_CONTEXT_ERROR_INVALID_STATE,
+                                 "<replace-body> should be caused on "
+                                 "<end-of-message> but was <start>");
+    gcut_assert_equal_error(expected_error, actual_error);
 }
 
 void
 test_progress (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
+    milter_server_context_end_of_message(context, NULL, 0);
+    pump_all_events();
+
+    cut_assert_true(milter_server_context_is_processing(context));
     milter_reply_encoder_encode_progress(encoder, &packet, &packet_size);
     write_data(packet, packet_size);
     cut_assert_equal_int(1, n_progresss);
     milter_assert_status(MILTER_STATUS_NOT_CHANGE);
+    cut_assert_true(milter_server_context_is_processing(context));
+}
+
+void
+test_progress_on_invalid_state (void)
+{
+    const gchar *packet;
+    gsize packet_size;
+
+    milter_reply_encoder_encode_progress(encoder, &packet, &packet_size);
+    write_data_without_error_handling(packet, packet_size);
+    cut_assert_equal_int(0, n_progresss);
+    cut_assert_equal_int(1, n_errors);
+
+    expected_error = g_error_new(MILTER_SERVER_CONTEXT_ERROR,
+                                 MILTER_SERVER_CONTEXT_ERROR_INVALID_STATE,
+                                 "<progress> should be caused on "
+                                 "<end-of-message> but was <start>");
+    gcut_assert_equal_error(expected_error, actual_error);
 }
 
 void
 test_quarantine (void)
 {
+    const gchar *packet;
+    gsize packet_size;
     const gchar reason[] = "infection";
 
+    milter_server_context_end_of_message(context, NULL, 0);
+    pump_all_events();
+
+    cut_assert_true(milter_server_context_is_processing(context));
     milter_reply_encoder_encode_quarantine(encoder, &packet, &packet_size, reason);
-    write_data(packet, packet_size);
+    write_data_without_error_handling(packet, packet_size);
     cut_assert_equal_int(1, n_quarantines);
     cut_assert_equal_string(reason, actual_quarantine_reason);
     milter_assert_status(MILTER_STATUS_QUARANTINE);
+    cut_assert_true(milter_server_context_is_processing(context));
+}
+
+void
+test_quarantine_on_invalid_state (void)
+{
+    const gchar *packet;
+    gsize packet_size;
+    const gchar reason[] = "infection";
+
+    milter_reply_encoder_encode_quarantine(encoder, &packet, &packet_size, reason);
+    write_data_without_error_handling(packet, packet_size);
+    cut_assert_equal_int(0, n_quarantines);
+    cut_assert_equal_int(1, n_errors);
+
+    expected_error = g_error_new(MILTER_SERVER_CONTEXT_ERROR,
+                                 MILTER_SERVER_CONTEXT_ERROR_INVALID_STATE,
+                                 "<quarantine> should be caused on "
+                                 "<end-of-message> but was <start>");
+    gcut_assert_equal_error(expected_error, actual_error);
 }
 
 void
 test_connection_failure (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     milter_reply_encoder_encode_connection_failure(encoder, &packet, &packet_size);
     write_data(packet, packet_size);
     cut_assert_equal_int(1, n_connection_failures);
@@ -856,6 +1219,9 @@ test_connection_failure (void)
 void
 test_shutdown (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     milter_reply_encoder_encode_shutdown(encoder, &packet, &packet_size);
     write_data(packet, packet_size);
     cut_assert_equal_int(1, n_shutdowns);
@@ -865,6 +1231,9 @@ test_shutdown (void)
 void
 test_skip (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     milter_server_context_set_state(context,
                                     MILTER_SERVER_CONTEXT_STATE_BODY);
 
@@ -876,15 +1245,20 @@ test_skip (void)
 }
 
 void
-test_skip_in_invalid_state (void)
+test_skip_on_invalid_state (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
+    milter_reply_encoder_encode_skip(encoder, &packet, &packet_size);
+    write_data_without_error_handling(packet, packet_size);
+    cut_assert_equal_int(0, n_skips);
+    cut_assert_equal_int(1, n_errors);
+
     expected_error = g_error_new(MILTER_SERVER_CONTEXT_ERROR,
                                  MILTER_SERVER_CONTEXT_ERROR_INVALID_STATE,
-                                 "Invalid state: start");
-    milter_reply_encoder_encode_skip(encoder, &packet, &packet_size);
-    write_data(packet, packet_size);
-    cut_assert_equal_int(0, n_skips);
-
+                                 "<skip> should be caused on "
+                                 "<body> but was <start>");
     cut_assert_false(milter_server_context_get_skip_body(context));
     gcut_assert_equal_error(expected_error, actual_error);
 }
@@ -893,8 +1267,9 @@ static gboolean
 cb_timeout (MilterServerContext *context, gpointer data)
 {
     gboolean *timeout_flag = data;
+
     *timeout_flag = TRUE;
-    gcut_string_io_channel_set_buffer_limit(channel, 0);
+    gcut_string_io_channel_set_buffer_limit(read_channel, 0);
     return FALSE;
 }
 
@@ -902,6 +1277,7 @@ static gboolean
 cb_timeout_detect (gpointer data)
 {
     gboolean *timeout_flag = data;
+
     *timeout_flag = TRUE;
     return FALSE;
 }
@@ -910,8 +1286,9 @@ static gboolean
 cb_io_timeout_detect (gpointer data)
 {
     gboolean *timeout_flag = data;
+
     *timeout_flag = TRUE;
-    gcut_string_io_channel_set_buffer_limit(channel, 0);
+    gcut_string_io_channel_set_buffer_limit(write_channel, 0);
     return FALSE;
 }
 
@@ -920,15 +1297,20 @@ test_reading_timeout (void)
 {
     gboolean timed_out_reading = FALSE;
 
+    return;
     milter_server_context_set_reading_timeout(context, 1);
     milter_server_context_helo(context, "delian");
 
     g_signal_connect(context, "reading-timeout",
                      G_CALLBACK(cb_timeout), &timed_out_reading);
-    g_timeout_add(500, cb_timeout_detect, &timed_out_before);
-    timeout_id = g_timeout_add(2000, cb_timeout_detect, &timed_out_after);
+    milter_event_loop_add_timeout(loop, 0.5,
+                                  cb_timeout_detect, &timed_out_before);
+    timeout_id = milter_event_loop_add_timeout(loop,
+                                               2,
+                                               cb_timeout_detect,
+                                               &timed_out_after);
     while (!timed_out_reading && !timed_out_after)
-        g_main_context_iteration(NULL, TRUE);
+        milter_event_loop_iterate(loop, TRUE);
     cut_assert_true(timed_out_before);
     cut_assert_true(timed_out_reading);
     cut_assert_false(timed_out_after);
@@ -937,62 +1319,44 @@ test_reading_timeout (void)
 void
 test_writing_timeout (void)
 {
-    GIOChannel *read_channel;
     gboolean timed_out_writing = FALSE;
 
-    read_channel = gcut_string_io_channel_new(NULL);
-    cut_take(read_channel, (CutDestroyFunction)g_io_channel_unref);
-    g_io_channel_set_encoding(read_channel, NULL, NULL);
-    gcut_string_io_channel_set_pipe_mode(read_channel, TRUE);
-    g_object_unref(reader);
-    reader = milter_reader_io_channel_new(read_channel);
-    milter_agent_set_reader(MILTER_AGENT(context), reader);
-    milter_agent_start(MILTER_AGENT(context), NULL);
-
+    return;
     milter_server_context_set_writing_timeout(context, 1);
     g_signal_connect(context, "writing-timeout",
                      G_CALLBACK(cb_timeout), &timed_out_writing);
-    g_timeout_add(500, cb_timeout_detect, &timed_out_before);
-    timeout_id = g_timeout_add(2000, cb_io_timeout_detect, &timed_out_after);
+    milter_event_loop_add_timeout(loop, 0.5,
+                                  cb_timeout_detect, &timed_out_before);
+    timeout_id = milter_event_loop_add_timeout(loop,
+                                               2,
+                                               cb_io_timeout_detect,
+                                               &timed_out_after);
 
-    g_io_channel_set_buffered(channel, FALSE);
-    gcut_string_io_channel_set_buffer_limit(channel, 1);
-    gcut_string_io_channel_set_pipe_mode(channel, TRUE);
+    g_io_channel_set_buffered(write_channel, FALSE);
+    gcut_string_io_channel_set_buffer_limit(write_channel, 1);
+    gcut_string_io_channel_set_pipe_mode(write_channel, TRUE);
+    g_io_channel_set_close_on_unref(write_channel, TRUE);
 
     milter_server_context_helo(context, "delian");
     while (!timed_out_writing && !timed_out_after)
-        g_main_context_iteration(NULL, TRUE);
+        milter_event_loop_iterate(loop, TRUE);
     cut_assert_true(timed_out_before);
     cut_assert_true(timed_out_writing);
     cut_assert_false(timed_out_after);
 }
 
-void
-test_invalid_state_error (void)
-{
-    const gchar from[] = "example@example.com";
-
-    expected_error = g_error_new(MILTER_SERVER_CONTEXT_ERROR,
-                                 MILTER_SERVER_CONTEXT_ERROR_INVALID_STATE,
-                                 "Invalid state: start");
-    milter_reply_encoder_encode_change_from(encoder,
-                                            &packet, &packet_size,
-                                            from, NULL);
-    write_data(packet, packet_size);
-    cut_assert_equal_int(1, n_errors);
-    gcut_assert_equal_error(expected_error, actual_error);
-}
-
 void
 test_busy_error (void)
 {
+    milter_server_context_helo(context, "delian");
+    pump_all_events();
+    milter_server_context_data(context);
+    milter_test_pump_all_events(loop);
+
     expected_error = g_error_new(MILTER_SERVER_CONTEXT_ERROR,
                                  MILTER_SERVER_CONTEXT_ERROR_BUSY,
                                  "previous command has been processing: "
                                  "helo -> data");
-
-    milter_server_context_helo(context, "delian");
-    milter_server_context_data(context);
     gcut_assert_equal_error(expected_error, actual_error);
 }
 
@@ -1000,9 +1364,11 @@ void
 test_no_busy_on_abort (void)
 {
     milter_server_context_helo(context, "delian");
+    pump_all_events();
     milter_server_context_abort(context);
+    pump_all_events();
     milter_server_context_quit(context);
-    gcut_assert_error(actual_error);
+    pump_all_events();
 }
 
 static void
@@ -1019,17 +1385,16 @@ pack (GString *buffer)
 void
 test_decode_error (void)
 {
+    g_string_append(buffer, "XXXX");
+    pack(buffer);
+    write_data_without_error_handling(buffer->str, buffer->len);
+
     expected_error = g_error_new(MILTER_AGENT_ERROR,
                                  MILTER_AGENT_ERROR_DECODE_ERROR,
                                  "Decode error: "
                                  "milter-decoder-error-quark:%u: "
                                  "unexpected reply was received: X",
                                  MILTER_DECODER_ERROR_UNEXPECTED_COMMAND);
-
-    g_string_append(buffer, "XXXX");
-    pack(buffer);
-    write_data(buffer->str, buffer->len);
-
     gcut_assert_equal_error(expected_error, actual_error);
 }
 
@@ -1038,17 +1403,24 @@ test_write_error (void)
 {
     GError *error = NULL;
 
-    g_io_channel_set_buffered(channel, FALSE);
-    g_io_channel_set_flags(channel, G_IO_FLAG_NONBLOCK, &error);
+    g_io_channel_set_buffered(write_channel, FALSE);
+    g_io_channel_set_flags(write_channel, G_IO_FLAG_NONBLOCK, &error);
     gcut_assert_error(error);
-    gcut_string_io_channel_set_limit(channel, 1);
+    gcut_string_io_channel_set_limit(write_channel, 1);
 
-    expected_error = g_error_new(MILTER_SERVER_CONTEXT_ERROR,
-                                 MILTER_SERVER_CONTEXT_ERROR_IO_ERROR,
-                                 "Failed to write to milter: "
-                                 "g-io-channel-error-quark:4: %s",
-                                 g_strerror(ENOSPC));
     milter_server_context_helo(context, "delian");
+    milter_test_pump_all_events(loop);
+
+    expected_error = g_error_new(MILTER_AGENT_ERROR,
+                                 MILTER_AGENT_ERROR_IO_ERROR,
+                                 "Output error: "
+                                 "%s:%d: failed to write: "
+                                 "%s:%d: %s",
+                                 g_quark_to_string(MILTER_WRITER_ERROR),
+                                 MILTER_WRITER_ERROR_IO_ERROR,
+                                 g_quark_to_string(G_IO_CHANNEL_ERROR),
+                                 G_IO_CHANNEL_ERROR_NOSPC,
+                                 g_strerror(ENOSPC));
     gcut_assert_equal_error(expected_error, actual_error);
 }
 
